function figure_handle = plot_flow_signals(sums,time_vector,signal_names)

% Assign an empty matrix to non provided input arguments to signal
% they will be assigned default values
if (nargin < 3), signal_names = []; end;

% The regions' names
region_names = fieldnames(sums);

% The number of regions
n_regions = length(region_names);

% The names of all signals in the variable 'sums'
all_signal_names = fieldnames(sums.(region_names{1}));

% If the input parameter 'signal_names' has not been provided,
% it defaults to all signals
if isempty(signal_names), signal_names = all_signal_names; end;

% The names of signals not to be plotted
ignored_signal_names = setdiff(all_signal_names,signal_names);

% The plot legends
legendas = { ...
	'x'         'Horizontal';
	'y'         'Vertical';
	'mag'       'Sum of magnitudes';
	'x_mag'     'Horz. Magnitude';
	'y_mag'     'Vert. Magnitude';
	'sum_mag'   'Magnitude of sum'};

% Plot the sums for all regions
figure_handle = figure;
for m=1:n_regions
	
	axes_handles(m) = subplot(n_regions,1,m);
	% signals = [sums.(region_names{m}).x sums.(region_names{m}).y sums.(region_names{m}).mag];
	% signals = struct2array(sums.(region_names{m}));
	signals = struct2array(orderfields(rmfield(sums.(region_names{m}),ignored_signal_names),signal_names));
	plot(time_vector,signals,'.-');
	ylabel(region_names{m},'Interpreter','none');
	
	% legend('Horizontal','Vertical','Magnitude');
	% [aux,indices] = ismember(fieldnames(sums.(region_names{m})),legendas(:,1));
	[aux,indices] = ismember(signal_names,legendas(:,1));
	legend(legendas(indices,2));
	if (m<n_regions), set(gca,'XTickLabel',[]); end;
	
end

% Link all axes along the horizontal direction
linkaxes(axes_handles,'x');

% xlabel('Time (s)');
xlabel(axes_handles(end),'Time (s)');

% --------------------------------------------------------------------------------------------- %