function group_data = read_flow_signals(file_name)

% Fetch information about the HDF file
file_info = hdf5info(file_name);

% The root group
group = file_info.GroupHierarchy;

% Read all data under the root group
group_data = read_group(group);

% --------------------------------------------------------------------------------------------- %

function group_data = read_group(group)

% The number of sub-groups in the group
n_groups = length(group.Groups);

% The number of data sets in the group
n_data_sets = length(group.Datasets);

% Loop over data sets
for m=1:n_data_sets
	
	% The name of the current data set
	data_set_name = group.Datasets(m).Name;
	
	% The name of the field in the structure 'group_data' where the current
	% data set will be stored
	indices = strfind(data_set_name,'/');
	field_name = data_set_name(indices(end)+1:end);
	
	% Read the current data set and store it in the variable 'group_data'
	group_data.(field_name) = hdf5read(group.Filename,data_set_name);
	
end

% Loop over sub-groups
for m=1:n_groups
	
	% The name of the current sub-group
	indices = strfind(group.Groups(m).Name,'/');
	group_name = group.Groups(m).Name(indices(end)+1:end);
	
	% Read the current sub-group and store it in the variable 'group_data'
	group_data.(group_name) = read_group(group.Groups(m));
	
end

% --------------------------------------------------------------------------------------------- %