function group_data = read_flow_signals(file_name)

% Fetch information about the HDF file
group_hierarchy = h5info(file_name);

% The HDF file name (taken from the structure returned by h5info())
file_name = group_hierarchy.Filename;

% Recursively read all data under the root group
group_data = read_group(file_name,group_hierarchy);

% --------------------------------------------------------------------------------------------- %

function group_data = read_group(file_name,group_hierarchy)

% The name of the current group
group_name = group_hierarchy.Name;

% The number of sub-groups in the group
n_groups = length(group_hierarchy.Groups);

% The number of data sets in the group
n_data_sets = length(group_hierarchy.Datasets);

% Loop over data sets
for m=1:n_data_sets
	
	% The name of the current data set
	data_set_name = group_hierarchy.Datasets(m).Name;
	
	% Read the current data set and store it in the variable 'group_data'
	group_data.(data_set_name) = h5read(file_name,fullfile(group_name,data_set_name));
	
end

% Loop over sub-groups
for m=1:n_groups
	
	% The name of the current sub-group
	sub_group_name = group_hierarchy.Groups(m).Name;
	[zzz,sub_group_name] = fileparts(sub_group_name);
	
	% Read the current sub-group and store it in the variable 'group_data'
	group_data.(sub_group_name) = read_group(file_name,group_hierarchy.Groups(m));
	
end

% --------------------------------------------------------------------------------------------- %