# -*- coding: utf-8 -*-

# This file started out as a copy of the file /home/adriano/devel/pyffmpeg/process_video.py
# on November 08, 2010. It is meant to be a GUI that will replace the current command line
# optical flow application (optical_flow_mac.py).

# This file started out as a copy of the file flow_gui_2.py on April 03, 2011. We'll be
# working on adding an image to the gui and letting the user add regions of interest
# and disinterest. In order to do that, we'll be looking at the Qt classes QRubberBand
# and QRect. Instead of using one pushbutton for the rois and another pushbutton for
# the rods, it may be a good idea to have a single pushbutton and then define the type
# of the region in the dialog that prompts for the region name.

# ----------------------------------------------------------------------------------------------- #

# Changelog:

# - v05 (2016/12/??) - added a checkbox ("Show plots") to the GUI so that it's now possible
#   to enable/disable the showing of plots at the end of the processing
#
# - v06 (2017/01/06) - code ported from PyQt4 to PyQt5

# ----------------------------------------------------------------------------------------------- #

import os, sys, time, numpy, tables
import flow_lib
# import opencv_lib_4_cv21 as flow_lib
# import flow_lib_no_pyffmpeg as flow_lib
# from PyQt4 import QtCore
# from PyQt4 import QtGui
from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5 import QtWidgets
# import pyffmpeg
import matplotlib.pyplot as plt
import xlwt
import notify.all as notify

# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

#This dictionary maps the number of channels in the Numpy image array on the appropriate
#Pixel format to be used when constructing the corresponding QImage object

try:
	image_conversion_map={
		1:QtGui.QImage.Format_Indexed8,
		3:QtGui.QImage.Format_RGB888,
		4:QtGui.QImage.Format_RGB32
		}
except:
	image_conversion_map={
		1:QtGui.QImage.Format_Indexed8,
		4:QtGui.QImage.Format_RGB32
		}

## These dictionaries are used when opening the FFMpegReader object
#TS_VIDEO_RGB24 = {'video1':(0, -1, {'pixel_format':pyffmpeg.PixelFormats.RGB24})}
#TS_VIDEO_BGR24 = {'video1':(0, -1, {'pixel_format':pyffmpeg.PixelFormats.BGR24})}
#TS_VIDEO_GRAY8 = {'video1':(0, -1, {'pixel_format':pyffmpeg.PixelFormats.GRAY8})}

# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

class GraphicsView(QtWidgets.QGraphicsView):

# ----------------------------------------------------------------------------------------------- #

#	def __init__(self,scene):
#
#		# Call the method in the base class
#		super(GraphicsView,self).__init__(scene)
#
#		# Disable both the horizontal and the vertical scrollbars. When drawing a
#		# region of interest/disinterest at one of the image's edges (in such a way
#		# that at least one of the region's edges is on top of one of the image's
#		# edges), the drawing of the region will cause scrollbars to appear. This
#		# happens in order to accomodate the extra pixels introduced in the scene
#		# when the region's borders are drawn. In our case, these borders have a
#		# thickness larger than one pixel, and the drawing is done in such a way
#		# that half the borderline is inside the region, and the other half is
#		# outside it. These scrollbars are distracting and the only consequence
#		# of removing them is that region borders at image edges won't be fully
#		# seen (they will be half seen, to be exact). This is a small price to
#		# pay for the removal of the annoying scrollbars.
#		self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
#		self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)

# ----------------------------------------------------------------------------------------------- #

	def mousePressEvent(self,event):

		# Call the method in the base class
		super(GraphicsView,self).mousePressEvent(event)

		# The point where the mouse button was pressed down
		self.parent().current_selection['press_point'] = event.pos()

# ----------------------------------------------------------------------------------------------- #

	def mouseReleaseEvent(self,event):

		# Call the method in the base class
		super(GraphicsView,self).mouseReleaseEvent(event)

		# The point where the mouse button was released
		self.parent().current_selection['release_point'] = event.pos()

		# The dialog for entering the region's properties
		region_properties_dialog = RegionPropertiesDialog(self.parent())

		# Run the region properties dialog
		region_properties_dialog.exec_()

# ----------------------------------------------------------------------------------------------- #

	def wheelEvent(self,event):

		# This function is reimplemented just in order to ignore mousewheel events.

		pass

# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

class RegionPropertiesDialog(QtWidgets.QDialog):

	def __init__(self,parent=None):

		# Call the method in the base class
		super(RegionPropertiesDialog, self).__init__(parent)

		# The region name and type
		self.region_name = None
		self.region_type = None

		# Create the dialog's gui
		self.create_gui()

# ----------------------------------------------------------------------------------------------- #

	def create_gui(self):

		# Set dialog title
		self.setWindowTitle("Define region")

		self.region_name_line_edit = QtWidgets.QLineEdit()
		self.region_type_combobox = QtWidgets.QComboBox()

		# The possible regions types
		self.region_types = ("Interest", "Disinterest")

		# Populate the flow methods combobox
		self.region_type_combobox.addItems(self.region_types)

		# QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel
		self.button_box = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)

		form_layout = QtWidgets.QFormLayout()
		form_layout.addRow("Region name",self.region_name_line_edit)
		form_layout.addRow("Region type",self.region_type_combobox)

		v_layout = QtWidgets.QVBoxLayout()
		v_layout.addLayout(form_layout)
		v_layout.addWidget(self.button_box)

		# Set the dialog's layout
		self.setLayout(v_layout)

		# Signals/slots connections
		self.button_box.accepted.connect(self.accept)
		self.button_box.rejected.connect(self.reject)
		self.region_name_line_edit.textChanged.connect(self.on_changed_region_name)
		self.region_type_combobox.currentIndexChanged.connect(self.on_changed_region_type)

		# We emit the combobox's currentIndexChanged() signal in order to get the
		# on_changed_region_type() slot executed (which, in turn, will cause the
		# on_changed_region_name() slot to be executed as well)
		self.region_type_combobox.currentIndexChanged.emit(self.region_type_combobox.currentIndex())

# ----------------------------------------------------------------------------------------------- #

	def accept(self):

		# Call the method in the base class
		super(RegionPropertiesDialog, self).accept()

		# Add the new region to the list of regions
		# self.parent().add_region(self)
		# self.parent().add_region(self.region_name, self.region_type)

		# Add the new region to the list of regions
		point_1 = self.parent().current_selection['press_point']
		point_2 = self.parent().current_selection['release_point']
		self.parent().add_region(point_1, point_2, self.region_name, self.region_type)

# ----------------------------------------------------------------------------------------------- #

	def on_changed_region_name(self):

		# Read the region name from the region properties dialog and update
		# the corresponding attribute
		self.region_name = unicode(self.region_name_line_edit.text())

# ----------------------------------------------------------------------------------------------- #

	def on_changed_region_type(self):

		# Read the region type from the region properties dialog and update
		# the corresponding attribute
		# self.region_type = unicode(self.region_type_combobox.currentText().toLower())
		# self.region_type = unicode(self.region_type_combobox.currentText().lower())
		self.region_type = unicode(self.region_type_combobox.currentText()).lower()

		# Reset the region name according to the region type
		self.__define_default_region_name()

# ----------------------------------------------------------------------------------------------- #

	def __define_default_region_name(self):

		# The number of regions of interest (regions of disinterest are not considered here)
		n_regions = [region["type"].lower() for region in self.parent().regions].count("interest")

		# Define the region name according to the active region type
		if (self.region_type.lower() == 'interest'):
			region_name = "region_%02g" % (n_regions+1)
		else:
			region_name = "Ignored"

		# Update the region name in the line edit widget
		self.region_name_line_edit.setText(region_name)

# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

class FlowGui(QtWidgets.QWidget):

	#----- Signals emitted by this class -----#

	# Signal emitted when the optical flow analysis is canceled
	analysis_canceled = notify.Signal()

# ----------------------------------------------------------------------------------------------- #

	def __init__(self,parent=None):

		# Call the parent class' constructor (Python doesn't do this automatically)
		super(FlowGui, self).__init__(parent)





# -------------------------------------------------------- #

# This is the constructor for the FlowCalculator's FlowGui class. I think we need
# many of those attributes here in the FlowAnalyzer's FlowGui class as well


#	def __init__(self,parent=None):
#
#		# Call the parent class' constructor (Python doesn't do this automatically)
#		super(FlowGui, self).__init__(parent)
#
#
#		# Get valid flow methods, output file types and video backends
#		# from the FlowApplication class in the flow_lib module
#		self.flow_methods      = flow_lib.FlowAnalysis.flow_methods
#		self.video_backends    = flow_lib.FlowAnalysis.video_backends
#		# self.output_file_types = flow_lib.FlowApplication.output_file_types
#		self.output_file_types = flow_lib.FlowApplication.flow_file_types
#
#		# In the final GUI, we will have to be able to define if we want or not an
#		# output file name and, if so, whether or not to use the default file name
#		self.input_file_name  = None
#		# self.output_file_name = None
#		self.output_file_name = ""
#
#		self.use_default_output_file_name = None
#
#		self.initial_frame = None
#		self.final_frame   = None
#
#		self.flow_method      = None
#		self.output_file_type = None
#		self.video_backend    = None
#
#		# Create the application's gui
#		self.create_gui()

# -------------------------------------------------------- #


		self.input_file_name  = None
		# self.output_file_name = None
		self.output_file_name = ""


		self.initial_frame = None
		self.final_frame   = None


		# self.save_flow_file_checkbox = None


		# Create the application's gui
		self.create_gui()

# ----------------------------------------------------------------------------------------------- #

	def create_gui(self):


		# A dictionary list with all the defined regions
		self.regions = []

		# A property containing the current selected area on the image pixmap
		self.current_selection = {"press_point" : None, "release_point" : None}


		# Construct widgets. Note that we don't need to specify the widgets' parent;
		# they will get their parent when we add them to our layout


#		# The dialog for entering the region's properties
#		self.region_properties_dialog = RegionPropertiesDialog(self)


		# The image scene
		# QGraphicsScene scene;
		self.image_scene = QtWidgets.QGraphicsScene(self)

		# The image view
		self.image_view = GraphicsView(self.image_scene)


		# Disable both the horizontal and the vertical scrollbars. When drawing a
		# region of interest/disinterest at one of the image's edges (in such a way
		# that at least one of the region's edges is on top of one of the image's
		# edges), the drawing of the region will cause scrollbars to appear. This
		# happens in order to accomodate the extra pixels introduced in the scene
		# when the region's borders are drawn. In our case, these borders have a
		# thickness larger than one pixel, and the drawing is done in such a way
		# that half the borderline is inside the region, and the other half is
		# outside it. These scrollbars are distracting and the only consequence
		# of removing them is that region borders at image edges won't be fully
		# seen (they will be half seen, to be exact). This is a small price to
		# pay for the removal of the annoying scrollbars.
		self.image_view.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
		self.image_view.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)


		# Widgets for the input file
		self.input_file_button     = QtWidgets.QPushButton("Open file")

		# Widgets for the regions file
		self.regions_file_button     = QtWidgets.QPushButton("Load regions")

		# Widgets for the initial frame
		self.initial_frame_label   = QtWidgets.QLabel("Initial Frame")
		self.initial_frame_spinbox = QtWidgets.QSpinBox()

		# Widgets for the final frame
		self.final_frame_label     = QtWidgets.QLabel("Final Frame")
		self.final_frame_spinbox   = QtWidgets.QSpinBox()

		# Widget for starting the computation
		self.run_button            = QtWidgets.QPushButton("Go")

		# Widget for choosing whether a figure with the flow signals should be shown on screen
		self.show_plots_checkbox = QtWidgets.QCheckBox ("Show plots")
		self.show_plots_checkbox.setChecked(True)

		# Widget for choosing whether an output flow file should be saved
		self.save_flow_file_checkbox = QtWidgets.QCheckBox ("Save flow file")
		# self.save_flow_file_checkbox.setChecked(True)

		# Widget for the video slider
		self.video_slider      = QtWidgets.QSlider(QtCore.Qt.Horizontal)

		# Widget for the number of the current frame
		self.current_frame_number_label     = QtWidgets.QLabel("0")

		# Initially, only the input file widgets are enabled. The remaining widgets
		# will be enabled once the input file has been defined
		self.regions_file_button.setEnabled(False)
		self.initial_frame_spinbox.setEnabled(False)
		self.final_frame_spinbox.setEnabled(False)
		self.run_button.setEnabled(False)
		self.video_slider.setEnabled(False)

		# An image containing the current video frame. At the time the GUI is
		# initialized, we don't have access to the actual video frame to be shown,
		# as that will be know only after the optical flow file has been loaded.
		# In the meantime, we show a generic 480x720 image as a placeholder, in
		# order to avoid a too small application window.
		# self.video_frame = QtGui.QPixmap(720,480)
		self.video_frame = QtGui.QPixmap(640,480)

		# The 'video_frame' pixmap above is uninitialized. We call its fill() method
		# to fill it with a white color
		self.video_frame.fill()

		# Assign the 'video_frame' pixmap to the image frame label
		# self.image_frame_label.setPixmap(self.video_frame)

		# Add the image pixmap to our scene. The variable 'self.video_frame'
		# then becomes a 'QGraphicsPixmapItem'
		# self.image_scene.addPixmap(self.video_frame)
		self.video_frame = self.image_scene.addPixmap(self.video_frame)
		# self.video_frame.setShapeMode(QtWidgets.QGraphicsPixmapItem.BoundingRectShape)

		# Now we can show view our scene
		self.image_view.show()

		# Set widgets' tooltips
		self.input_file_button.setToolTip("Select the input optical flow file.")
		self.initial_frame_spinbox.setToolTip("The first frame of the video segment to be processed.\n"
			"Minimum value: 0")
		self.final_frame_spinbox.setToolTip("The last frame of the video segment to be processed.\n"
			"Maximum value: not defined yet.")
		self.run_button.setToolTip("Process the optical flow file.")

#		# Arrange widgets on a layout. This will re-parent all widgets in the layout
		v_layout = QtWidgets.QVBoxLayout()
		v_layout.addWidget(self.input_file_button)
		v_layout.addStretch(1)
		v_layout.addWidget(self.regions_file_button)
		v_layout.addStretch(1)
		v_layout.addWidget(self.initial_frame_label)
		v_layout.addWidget(self.initial_frame_spinbox)
		v_layout.addWidget(self.final_frame_label)
		v_layout.addWidget(self.final_frame_spinbox)
		v_layout.addStretch(1)
		v_layout.addWidget(self.show_plots_checkbox)
		v_layout.addWidget(self.save_flow_file_checkbox)
		v_layout.addStretch(1)
		v_layout.addWidget(self.run_button)
		v_layout.addStretch(10)

		# Arrange widgets on a layout. This will re-parent all widgets in the layout
		layout = QtWidgets.QGridLayout()

		# The second column (column 1) is used only to keep a minimum space between
		# its adjacent columns
		layout.setColumnMinimumWidth(1,20)

		# Now, we add the widgets to the layout
		layout.addWidget(self.image_view,0,0)

		# TODO: THE SLIDER WIDTH CHANGES WHEN THE FIGURE SIZE IS CHANGED. WE HAVE
		# TO CHANGE THIS BEHAVIOUR, BECAUSE WE WANT THE SLIDER WIDTH TO BE ALWAYS
		# THE SAME AS THE IMAGE FRAME (WHICH DOESN'T CHANGE WHEN THE APPLICATION
		# WINDOW IS RESIZED)
		# layout.addWidget(self.video_slider,7,0)
		# layout.addWidget(self.video_slider,11,0)
		layout.addWidget(self.video_slider,1,0)

		layout.addLayout(v_layout,0,1)

		layout.addWidget(self.current_frame_number_label,1,1)


		# Set the window layout
		self.setLayout(layout)

#		# Signals/slots connections
		self.input_file_button.clicked.connect(self.choose_input_file)
#		self.output_file_button.clicked.connect(self.choose_output_file)
#		self.run_button.clicked.connect(self.run_flow_analysis)


		self.regions_file_button.clicked.connect(self.load_regions_file)


#		self.save_flow_file_checkbox.stateChanged.connect(self.on_changed_save_flow_file_checkbox)

		# self.run_button.clicked.connect(self.process_flow_file)
		self.run_button.clicked.connect(self.run)

		# TODO: WE CURRENTLY UPDATE THE FRAME NUMBER LABEL WHENEVER THE VIDEO
		# SLIDER CHANGES. THE CORRECT BEHAVIOUR WOULD BE TO UPDATE THE LABEL
		# WHENEVER A NEW VIDEO FRAME IS SHOWN ON SCREEN. WE MAY WANT TO CREATE
		# A NEW SIGNAL/SLOT CONNECTION TO UPDATE THE SLIDER WHENEVER A NEW VIDEO
		# FRAME IS SHOWN TOO. IF WE DO THAT, THE FRAME NUMBER LABEL WILL BE ALSO
		# UPDATED, AS IT IS TRIGGERED BY A SLIDER CHANGE
		self.initial_frame_spinbox.valueChanged.connect(self.on_changed_spinbox)
		self.final_frame_spinbox.valueChanged.connect(self.on_changed_spinbox)
		self.video_slider.valueChanged.connect(self.process_frame)
		self.video_slider.valueChanged.connect(self.current_frame_number_label.setNum)

		# Set the window title
		self.setWindowTitle("Optical Flow Reader")

#		# Disable window resizing in the vertical direction. However, we can
#		# only get the correct window height after running self.show()
#		self.show()
#		self.setFixedHeight(self.height())

# ----------------------------------------------------------------------------------------------- #

	def load_regions_file(self):

#		# The previous input file. We compare this to the current file chosen
#		# by the user. If the files are the same, then there's nothing to do.
#		# previous_input_file = str(self.input_file_line_edit.text())
#		previous_input_file = str(self.input_file_label.text())

		# The current directory
		current_dir = os.getcwd()

		# The formats filter
		# formats = "Video Files (*.mov *.avi *.wmv *.mpeg *.mpg)"
		# formats = "Optical flow files (*.bin *.hdf)"
		formats = "Excel Files (*.xls *.xlsx);; All Files (*)"

		# Open a file dialog for choosing the input file
		# self.input_file_name = QtWidgets.QFileDialog.getOpenFileName(self,"Load input video",current_dir,formats)
		# self.input_file_name = QtWidgets.QFileDialog.getOpenFileName(self,"Load flow regions file",current_dir,formats)
		regions_file_name = QtWidgets.QFileDialog.getOpenFileName(self,"Load regions file",current_dir,formats)
		# self.input_file_name = str(self.input_file_name)
		# self.input_file_name = unicode(self.input_file_name)
		# regions_file_name = unicode(regions_file_name)
		regions_file_name = unicode(regions_file_name[0])

		# If no file name has been provided (because the user cancelled the process)
		# then there's nothing to do
		if self.input_file_name == "":
			return

		# Create a FlowRegions object
		frame_size = (self.input_video.width, self.input_video.height)
		flow_regions = flow_lib.FlowRegions(frame_size)

		# Load regions from the regions file
		flow_regions.load_from_file(regions_file_name)

		# Concatenate regions of interest and disinterest into a single list.
		# This allows us to use a single loop below to loop over all regions,
		# regardless of their types
		regions = flow_regions.rois + flow_regions.rods

		# A list of region types corresponding to the 'regions' list above
		region_types = ['interest']*len(flow_regions.rois) + ['disinterest']*len(flow_regions.rods)

		# Loop over regions
		for m in range(len(regions)):

			# The region name and type
			region_name = regions[m]['name']
			region_type = region_types[m]

			# The points defining the region
			point_1 = QtCore.QPoint(regions[m]['position'][0], regions[m]['position'][1])
			point_2 = QtCore.QPoint(regions[m]['position'][2], regions[m]['position'][3])

			# Add the region to the list of regions
			self.add_region(point_1, point_2, region_name, region_type)

# ----------------------------------------------------------------------------------------------- #

	def choose_input_file(self):

#		# The previous input file. We compare this to the current file chosen
#		# by the user. If the files are the same, then there's nothing to do.
#		# previous_input_file = str(self.input_file_line_edit.text())
#		previous_input_file = str(self.input_file_label.text())

		# The current directory
		current_dir = os.getcwd()

		# The formats filter
		# formats = "Video Files (*.mov *.avi *.wmv *.mpeg *.mpg)"
		# formats = "Optical flow files (*.bin *.hdf)"
		formats = "Video Files (*.mov *.avi *.wmv *.mpeg *.mpg);; Flow Files (*.bin *.hdf);; All Files (*)"

		# Open a file dialog for choosing the input file
		# self.input_file_name = QtWidgets.QFileDialog.getOpenFileName(self,"Load input video",current_dir,formats)
		self.input_file_name = QtWidgets.QFileDialog.getOpenFileName(self,"Load optical flow file",current_dir,formats)
		# self.input_file_name = str(self.input_file_name)
		# self.input_file_name = unicode(self.input_file_name)
		self.input_file_name = unicode(self.input_file_name[0])

		# If no file name has been provided (because the user cancelled the process)
		# then there's nothing to do
		if self.input_file_name == "":
			return

#		# If the current file chosen by the user is the same as before, then
#		# there's nothing to do
#		if input_file == previous_input_file:
#			return

		# Update the widget "input_file_line_edit" with the path to the input file
		# self.input_file_line_edit.setText(input_file)
		# self.input_file_label.setText(os.path.basename(input_file))
		# self.input_file_label.setText(input_file)

		# Update the window title with the name of the opened flow file
		window_title = self.windowTitle() + " - " + self.input_file_name
		self.setWindowTitle(window_title)

		# Now that the input file has been defined, we can enable the widgets
		# that were initially disabled
#		self.output_file_button.setEnabled(True)
		self.regions_file_button.setEnabled(True)
		#self.output_file_line_edit.setEnabled(True)
#		self.input_file_label.setEnabled(True)
#		self.output_file_label.setEnabled(True)
		self.initial_frame_spinbox.setEnabled(True)
		self.final_frame_spinbox.setEnabled(True)
#		self.method_combobox.setEnabled(True)
		self.run_button.setEnabled(True)
		self.video_slider.setEnabled(True)



#		# A dictionary mapping file extensions to the appropriate flow file constructor
#		extension_map = {".bin": flow_lib.FlowFileBinary, ".hdf": flow_lib.FlowFileHDF}
#
#		# The list of valid extensions
#		valid_extensions = extension_map.keys()
#
#		# Check if all files have a valid extension
#		(zzz,extension) = os.path.splitext(input_file)
#		if extension not in valid_extensions:
#			error_message = "%s: Invalid file type" % input_file
#			raise Exception(error_message)
#
#		# Create a flow file object
#		flow_class = extension_map[extension]
#		# flow_file = flow_class(None,input_file,"r")
#		# self.input_flow_file = flow_class(None,input_file,"r")
#		self.input_flow_file = flow_class(input_file,"r",None)



		# Initialize the names of the input video and flow files
		input_video_file_name = None
		input_flow_file_name  = None

		# Flow file extensions
		# flow_file_extensions = (".bin", ".hdf")
		flow_file_extensions = [file_type['file_extension'] for file_type in flow_lib.FlowApplication.flow_file_types]

		# The extension of the input file
		(zzz,input_file_extension) = os.path.splitext(self.input_file_name)

		# The input file type (either "video" or "flow")
		if input_file_extension.lower() in flow_file_extensions:
			input_file_type = "flow"
		else:
			input_file_type = "video"


		if (input_file_type.lower() == "video"):

			# The name of the input video file
			input_video_file_name = self.input_file_name


		if (input_file_type.lower() == "flow"):

			# The name of the input optical flow file
			input_flow_file_name = self.input_file_name

			# Determine the class to be used to open the optical flow file based
			# on the file's extension
			if (input_file_extension.lower() == ".bin"):
				self.input_flow_file = flow_lib.FlowFileBinary(input_flow_file_name,"r",None)
			elif (input_file_extension.lower() == ".hdf"):
				self.input_flow_file = flow_lib.FlowFileHDF(input_flow_file_name,"r",None)
			else:
				error_message = "Invalid flow file: %s" % input_flow_file_name
				raise Exception(error_message)


			# Open the flow file
			# flow_files[m].open_file()
			self.input_flow_file.open_file()

			# Read the file header. This must be done before reading any flow frames
			# flow_files[m].read_file_header()
			self.input_flow_file.read_file_header()

			# The input video file used in creating the optical flow file
			# input_video_file = self.input_flow_file.file_header['input_video_file']
			input_video_file_name = self.input_flow_file.file_header['input_video_file']

			# The directories for the video file (the one used for creating the
			# optical flow file we just opened) and for the optical flow file.
			# We search for the input video file in each of these directories.
			(input_flow_dir,zzz) = os.path.split(input_flow_file_name)
			(input_video_dir,input_video_file_short_name) = os.path.split(input_video_file_name)

			# List of directories where to search for the input video file
			# dir_list = [input_video_dir, input_flow_dir]
			dir_list = [input_video_dir, input_flow_dir, os.path.curdir]

			# We initially set the boolean 'input_video_file_exists' to false.
			# We then loop over the various directories where we'll look for
			# the input video file and, if it is found in any of them, we set
			# this boolean to true
			input_video_file_exists = False

			# Search for the input video file in all directories listed in 'dir_list'
			for dir_name in dir_list:
				input_video_file_name = os.path.join(dir_name,input_video_file_short_name)
				if os.path.exists(input_video_file_name):
					input_video_file_exists = True
					break

			# If the input video file could not be located anywhere, we throw an exception
			if not input_video_file_exists:
				error_message = "Could not locate the input video file. Giving up..."
				raise Exception(error_message)




		# TODO: DO WE NEED TO CHECK IF THERE ALREADY IS AN INPUT_VIDEO
		# OBJECT AND DELETE IT IF SO?
		# Construct the MovieReader object
		# self.input_video = flow_lib.MovieReaderPyffmpeg(input_video_file_name)
		self.input_video = flow_lib.MovieReaderOpencv(input_video_file_name)





		# IMPORTANT: I THINK IT WOULD BE VERY USEFUL TO CREATE A FLOWSOURCE()
		# CLASS THAT COULD BE USED TO ACCESS BOTH VIDEO AND FLOW FILES WITH
		# THE SAME SET OF ATTRIBUTES





		# TODO: THIS SCALES THE IMAGE SO THAT IT FITS IN THE IMAGE VIEW WIDGET,
		# BUT THAT'S STILL NOT WHAT WE WANT
		# self.image_view.fitInView(0, 0, self.input_video.width, self.input_video.height)

#		# Initialize the initial and final video frames with zero and
#		# the last frame, respectively
#		# initial_frame = 0
#		# final_frame = self.input_video.n_frames-1
#		initial_frame = self.input_flow_file.file_header['initial_frame']
#		final_frame = self.input_flow_file.file_header['final_frame']
#		# final_frame = self.input_flow_file.file_header['final_frame'] - 1



		# The initial and final frames
		if (input_file_type.lower() == "video"):
			initial_frame = 0
			final_frame   = self.input_video.n_frames-1
		else:
			initial_frame = self.input_flow_file.file_header['initial_frame']
			final_frame   = self.input_flow_file.file_header['final_frame']
			# final_frame = self.input_flow_file.file_header['final_frame'] - 1



		# Set the spinboxes' ranges
		self.initial_frame_spinbox.setRange(initial_frame,final_frame)
		self.final_frame_spinbox.setRange(initial_frame,final_frame)

		# Set limits on the slider widget
		self.video_slider.setMinimum(initial_frame)
		self.video_slider.setMaximum(final_frame)

		# Initialize the spinboxes' values with the first and last video frames
		self.initial_frame_spinbox.setValue(initial_frame)
		self.final_frame_spinbox.setValue(final_frame)

		# Update the final frame tooltip
		tooltip = "The last frame of the video segment to be processed.\nMaximum value: %g" % final_frame
		self.final_frame_spinbox.setToolTip(tooltip)

		# Set the observer function of the video reader object. This function will
		# be called everytime a video frame is read
		# self.input_video.input_video.set_observer(self.show_image)
		self.input_video.video_frame_read.connect(self.show_image_2)


		# Read the initial frame in the video file. Because we have set the method
		# show_image() as the observer function, this will cause the initial frame
		# of the video to be shown on the screen
		self.input_video.read_frame(initial_frame)

		# The widget's contents margins (left, top, right, bottom)
		frame_margins = self.image_view.getContentsMargins()

		# The frame width and height to be used to resize the QGraphicsView object
		# (self.image_view). Important: this *must* take the contents margins into
		# account
		frame_width = self.video_frame.pixmap().width() + frame_margins[0] + frame_margins[2]
		frame_height = self.video_frame.pixmap().height() + frame_margins[1] + frame_margins[3]


#		print self.image_view.contentsRect()
#		print self.image_view.frameGeometry()

		# Resize the the QGraphicsView object (self.image_view) in order to accomodate
		# the read image.
		# TODO: There must be a better way of doing this (instead of doing it explicitly).
		# What is it???
		self.image_view.resize(frame_width,frame_height)


#		print self.image_view.contentsRect()
#		print self.image_view.frameGeometry()


		# Initialize the video slider to the initial flow frame. This is important
		# when we open a flow file, play with the slider to browse the video frames
		# and, after that, decide to open a new flow file. If we don't reset the
		# video slider here, it will be showing whatever the current frame was
		# in the previous flow file, whereas the current frame in the new opened
		# file is 'initial_frame'
		self.video_slider.setValue(initial_frame)

		# Set the video slider width to the same as the image frame's
		self.video_slider.setFixedWidth(self.image_view.width())

		# Set the drag mode in our scene view to RubberBandDrag. This allows
		# us to define rectangular regions of interest/disinterest
		self.image_view.setDragMode(QtWidgets.QGraphicsView.RubberBandDrag)

		# Ensure that the size of the viewport is the same size as the image shown
		self.image_view.setMaximumWidth(self.image_view.frameSize().width())
		self.image_view.setMaximumHeight(self.image_view.frameSize().height())
		self.image_view.setMinimumWidth(self.image_view.frameSize().width())
		self.image_view.setMinimumHeight(self.image_view.frameSize().height())

# ----------------------------------------------------------------------------------------------- #

	def on_mouse_release_event(self,event):

		print "Hello"

# ----------------------------------------------------------------------------------------------- #

	def process_frame(self):


		# IMPORTANT: IN ORDER TO GET ONLY THE VIDEO FRAME CONTENTS, WE SHOULD CALL THE
		# METHOD get_next_frame() OF THE VIDEO_TRACK OBJECT AND NOT OF THE MEDIA_READER
		# OBJECT


#		# TODO: When reading frame N, we only have to seek to that frame if we're
#		# not coming from frame N-1
#		frame_number = self.slider.value()
#		#print "Reading frame %g" % frame_number
#		self.video_track.seek_to_frame(frame_number)
#		self.media_reader.get_next_frame()

		frame_number = self.video_slider.value()
		self.input_video.read_frame(frame_number)

# ----------------------------------------------------------------------------------------------- #

#	def show_image(self,array):
#
#		# Important: this method works with MovieReaderPyffmpeg objects
#
#		# The number of rows, columns and channels in the image array
#		(n_rows, n_cols, n_channels) = array.shape
#
#		# Convert the image array (which is a numpy.ndarray object) into a QImage object
#		aux = array.astype(numpy.uint8).copy('C')
#		image = QtGui.QImage(aux.data,n_cols,n_rows,image_conversion_map[n_channels])
#
#		#Update the image in the GUI (note that we have to convert the QImage object
#		#into a QPixmap object in order to be able to show it on the QLabel widget)
#		# self.image_frame.setPixmap(QtGui.QPixmap(image))
#		# self.image_frame_label.setPixmap(QtGui.QPixmap(image))
#		self.video_frame.setPixmap(QtGui.QPixmap(image))

# ----------------------------------------------------------------------------------------------- #

	def show_image_2(self,frame,frame_number):

		# Important: this method works with MovieReaderOpencv objects

		# Convert the frame image (which is an iplimage object) into a QImage object
		image = QtGui.QImage(frame.tostring(),frame.width,frame.height,image_conversion_map[frame.nChannels])
		image = image.rgbSwapped()

		#Update the image in the GUI (note that we have to convert the QImage object
		#into a QPixmap object in order to be able to show it on the QLabel widget)
		# self.image_frame_label.setPixmap(QtGui.QPixmap(image))
		self.video_frame.setPixmap(QtGui.QPixmap(image))

# ----------------------------------------------------------------------------------------------- #

	def on_changed_spinbox(self):

		# The current initial and final frames, gotten from the spinboxes
		self.initial_frame = self.initial_frame_spinbox.value()
		self.final_frame = self.final_frame_spinbox.value()

		# Very important: This method relies on self.final_frame_spinbox.value()
		# being greater than self.initial_frame_spinbox.value(). When these two
		# widgets are created, their values are initially zero. At that point,
		# if we try to set the value of the initial frame spinbox to something
		# greater than zero (this happens when the inital frame is greater than
		# zero), we'll have a situation where the initial frame is greater than
		# the final frame, and as a consequence this method will corrupt the
		# spinbox ranges. The easy way of taking care of this is to just check
		# if the value in the final frame spinbox is greater than the value in
		# the initial frame spinbox. If not, we just skip the code below that
		# adjusts the spin boxes ranges.
		if (self.final_frame <= self.initial_frame):
			return

		# Update the constraint on the spinboxes' ranges
		self.initial_frame_spinbox.setMaximum(self.final_frame-1)
		self.final_frame_spinbox.setMinimum(self.initial_frame+1)

# ----------------------------------------------------------------------------------------------- #

#	def on_changed_save_flow_file_checkbox(self,state):
#
#
#
#		print "Ola"
#
#
#		self.save_flow_file_checkbox

# ----------------------------------------------------------------------------------------------- #

	# def add_region(self,region_properties_dialog):
	# def add_region(self, region_name, region_type):
	def add_region(self, point_1, point_2, region_name, region_type):

		# The x and y coordinates of the region's rectangle
		# x_values = [self.current_selection[key].x() for key in self.current_selection.keys()]
		# y_values = [self.current_selection[key].y() for key in self.current_selection.keys()]
		x_values = [point_1.x(), point_2.x()]
		y_values = [point_1.y(), point_2.y()]

		# The coordinates of the region's top left (x_min, y_min) and bottom right
		# (x_max, y_max) corners
		min_x = min(x_values)
		min_y = min(y_values)
		max_x = max(x_values)
		max_y = max(y_values)

		# The region's rectangle
		delta_x = max_x - min_x
		delta_y = max_y - min_y
		rectangle = QtCore.QRectF(min_x,min_y,delta_x,delta_y)

		# Ensure that the region's rectangle is within the image frame
		rectangle = self.video_frame.boundingRect().intersected(rectangle)

		# The region name and type
		# region_name = region_properties_dialog.region_name
		# region_type = region_properties_dialog.region_type

		# A dictionary with the current region's properties
		region = {}
		region["region_widget"] = self.image_scene.addRect(rectangle)
		region["text_widget"] = self.image_scene.addSimpleText(region_name)
		region["name"] = region_name
		region["type"] = region_type
#		region["position"] = rectangle
		region["position"] = (rectangle.left(), rectangle.top(), rectangle.right(), rectangle.bottom())


#		# IMPORTANT!!! POSITION SPECIFIED AS [XI YI XF YF]. VERY IMPORTANT: THE UPPER
#		# LIMIT IS NOT INCLUDED (THAT'S HOW PYTHON WORKS!!!)
#
#		(rectangle.left(), rectangle.top(), rectangle.right(), rectangle.bottom())


		# The possible pen colors
		pen_colors = {"interest" : QtCore.Qt.green, "disinterest" : QtCore.Qt.red}

		# The pen color used to draw the current region's borders
		pen_color = pen_colors[region["type"].lower()]

		# The pen color used to draw the region's borders
		pen_width = 4

		# The font size used to write the region's name
		font_size = 12

		# Set the color of the current region borders according to the region type
		region["region_widget"].setPen(QtGui.QPen(pen_color,pen_width))

		# Set the color of the region's text label
		region["text_widget"].setBrush(QtGui.QBrush(pen_color))

		# Set the position of the region's text label
		region["text_widget"].setX(rectangle.x()+5)
		region["text_widget"].setY(rectangle.y()+5)

		# Set the font of the region's text label
		text_font = region["text_widget"].font()
		text_font.setPointSize(font_size)
		text_font.setWeight(QtGui.QFont.Bold)
		# text_font.setStyleHint(QtGui.QFont.SansSerif)
		region["text_widget"].setFont(text_font)

		# Append the new region of the list of regions (self.regions)
		self.regions.append(region)


# ----------------------------------------------------------------------------------------------- #

	def create_progress_bar(self,flow_app):

		n_flow_frames = self.final_frame - self.initial_frame
		self.progress_bar = QtWidgets.QProgressDialog(self)
		self.progress_bar.setMinimumDuration(2000)
		self.progress_bar.setLabelText("Processing frames...")
		# self.progress_bar.setRange(0, n_flow_frames - 1)
		self.progress_bar.setRange(self.initial_frame, self.final_frame - 1)
		self.progress_bar.setWindowModality(QtCore.Qt.WindowModal)


		# ----- Signal/slot connections ----- #

		# This connection allows the flow analysis status to be updated
		# when the analysis is canceled through the progress bar
		# self.analysis_canceled.connect(flow_analysis.update_analysis_status)
		self.analysis_canceled.connect(flow_app.analysis_canceled)

		# This connection allows the progress bar to be updated every time
		# a flow frame is computed
		# flow_analysis.flow_frame_computed.connect(self.update_progress_bar)
		# flow_app.flow_frame_computed.connect(self.update_progress_bar)
		flow_app.flow_frame_processed.connect(self.update_progress_bar)

# ----------------------------------------------------------------------------------------------- #

	def update_progress_bar(self,flow_frame,frame_number):

		# Update the progress bar
		self.progress_bar.setValue(frame_number)

		# Check whether the cancel button has been clicked. If so,
		# we emit the signal 'analysis_canceled'
		if (self.progress_bar.wasCanceled()):
			self.analysis_canceled(True)

# ----------------------------------------------------------------------------------------------- #



	def run(self):


#		input_file,								self.input_file_name
#		output_file,							self.output_file_name
#		initial_frame,						self.initial_frame
#		final_frame,							self.final_frame
#		flow_method="hs",					self.flow_method
#		flow_depth=16,						NOT SPECIFIED IN THE GUI
#		video_backend="opencv",			self.video_backend
#		output_file_type="hdf"				self.output_file_type



#		flow_regions = self.regions
#		flow_regions = []


		# The flow regions
		if (self.regions == []):

			flow_regions = []

		else:

			# Create a FlowRegions object
			frame_size = (self.input_video.width, self.input_video.height)
			flow_regions = flow_lib.FlowRegions(frame_size)

			# Add regions to the object 'flow_regions'
			for region in self.regions:
				flow_regions.add_region(region["name"], region["type"], region["position"])


		# The initial and final frames, gotten from the spinboxes. Please,
		# note that these values may be different from those in the optical
		# flow file. The values in the optical flow file tells us the range
		# of *video* frames used in computing the optical flow file. The
		# values in the spinboxes tells us the range of *optical flow*
		# frames to be analyzed
		initial_frame = self.initial_frame_spinbox.value()
		final_frame = self.final_frame_spinbox.value()



		if self.save_flow_file_checkbox.isChecked():
			self.output_file_name = ""
		else:
			self.output_file_name = None



#		# Construct the FlowApplication object and attach it to the FlowGui object
#		flow_app = flow_lib.FlowApplication(
#			self.input_flow_file,
#			flow_regions,
#			None,
#			initial_frame,
#			final_frame,
#			None,
#			None,
#			None,
#			None)


#		# Construct the FlowApplication object and attach it to the FlowGui object
#		flow_app = flow_lib.FlowApplication(
#			self.input_file_name,
#			flow_regions,
#			self.output_file_name,
#			self.initial_frame,
#			self.final_frame,
#			self.flow_method["name"],
#			16,
#			self.video_backend["name"],
#			self.output_file_type["name"])


		# Construct the FlowApplication object and attach it to the FlowGui object
		flow_app = flow_lib.FlowApplication(
			self.input_file_name,
			flow_regions,
			self.output_file_name,
			self.initial_frame,
			self.final_frame,
			None,
			16,
			None,
			None)





		# Create the progress bar
		self.create_progress_bar(flow_app)


##		if self.show_plots_checkbox.isChecked():
##			show_figure = True
##		else:
##			show_figure = False


		# Run the flow application
		# flow_app.run()
		show_plots = self.show_plots_checkbox.isChecked()
		flow_app.run(show_plots = show_plots)

# ----------------------------------------------------------------------------------------------- #

	def process_flow_file(self):


		# NOTE: THIS METHOD STARTED OUT AS A SIMPLE COPY OF THE METHOD
		# read_flow_file() IN THE FILE optical_flow_cv21_oo.py
		# TODO: I THINK IT WOULD BE A GOOD IDEA TO ADD THIS METHOD TO THE
		# MODULE OPENCV_LIB_4_CV21. THIS WOULD BE A METHOD THAT WOULD TAKE
		# THE REGIONS OF INTEREST AND/OR DISINTEREST AND RETURN ALL THE
		# SIGNALS COMPUTED FOR EACH OF THE REGIONS.

		# IMPORTANT!!! POSITION SPECIFIED AS [XI YI XF YF]. VERY IMPORTANT: THE UPPER
		# LIMIT IS NOT INCLUDED (THAT'S HOW PYTHON WORKS!!!)

#		# This region was obtained by loading the file 'CL1JD1Test_test_regions.mat'
#		# roi_1 = {'type': 'roi', 'name': 'face', 'position': [242, 29, 600, 362]}
#		# roi_1 = {'type': 'roi', 'name': 'face', 'position': [241, 28, 599, 361]}
#		roi_1 = {'type': 'roi', 'name': 'frame', 'position': [0, 0, 720, 480]}
#
#		# rods = [rod_1, rod_1, rod_1]
#		rods = numpy.array([])
#		# rois = [roi_1, roi_2, roi_3]
#		rois = [roi_1]


		# The regions of interest and disinterest
		rois = [region for region in self.regions if (region["type"].lower() == "interest")]
		rods = [region for region in self.regions if (region["type"].lower() == "disinterest")]


		# TODO: WHEN NO REGIONS OF INTEREST HAVE BEEN DEFINED, WE HAVE TO PRESENT
		# THE USER WITH A POPUP WINDOW LETTING HIM KNOW THAT WE'RE GOING TO CREATE
		# A DEFAULT ROI ENCOMPASSING THE WHOLE SCENE

		# If no regions of interest have been defined, we define a default one
		# containing the whole frame
		if not rois:

			# A dictionary with the current region's properties
			region = {}
			region["region_widget"] = None
			region["text_widget"] = None
			region["name"] = "frame"
			region["type"] = "interest"
			region["position"] = (0, 0, self.input_video.width, self.input_video.height)

			# Append the new region of the list of regions of interest (rois)
			rois.append(region)

		# The initial and final frames, gotten from the spinboxes. Please,
		# note that these values may be different from those in the optical
		# flow file. The values in the optical flow file tells us the range
		# of *video* frames used in computing the optical flow file. The
		# values in the spinboxes tells us the range of *optical flow*
		# frames to be analyzed
		initial_frame = self.initial_frame_spinbox.value()
		final_frame = self.final_frame_spinbox.value()

		# The number of optical flow frames to be analyzed
		n_frames = final_frame - initial_frame

		# Initialize the variable 'sums', where the summed flow within each region
		# of interest will be stored
		sums = [None]*n_frames

		# Define a progress bar
		progress_bar = QtWidgets.QProgressDialog(self)
		progress_bar.setMinimumDuration(2000)
		progress_bar.setLabelText("Processing frames...")
		progress_bar.setRange(initial_frame,final_frame)
		progress_bar.setWindowModality(QtCore.Qt.WindowModal)

		# Just a shortcut to the flow file object
		flow_file = self.input_flow_file

		# Start our stopwatch
		start_time = time.time()

		# Read all flow frames in the bin file
		for frame_number in range(initial_frame,final_frame):

			indice = frame_number - initial_frame

			progress_bar.setValue(frame_number)

			if (progress_bar.wasCanceled()):
				# break
				return

			# flow_file.read_next_frame()
			# flow_file.read_frame(frame_number)
			flow_file.read_frame(indice)
			flow_file.remove_rods(rods)
			# sums[m] = flow_file.process_rois(rois)
			sums[indice] = flow_file.process_rois(rois)
			# print "Reading frame %g" % flow_file.frame_number
			print "Reading frame %g" % frame_number


		# Finalize the progress bar and close it
		progress_bar.setValue(final_frame)

		# Re-organize the contents of the variable 'sums' into the variable 'signals'
		signals = self.__rearrange_motion_signals(sums,initial_frame,final_frame)

		# Stop our stopwatch and print out the elapsed time
		elapsed_time = (time.time() - start_time)
		print "Elapsed time = %g " % elapsed_time

		# The file names where to save the motion signals
		(base_file_name,extension) = os.path.splitext(flow_file.file_name)
		pdf_file_name   = "%s_motion_signals.pdf" % base_file_name
		hdf_file_name   = "%s_motion_signals.hdf" % base_file_name
		excel_file_name = "%s_motion_signals.xls" % base_file_name

		# The time vector. If we don't explicitly set the type to float here,
		# we get a sequence of integers and, if the frame rate happens to be
		# an integer as well, we end up with a sequence of integers for the
		# time vector, which is wrong.
		frame_rate  = self.input_flow_file.file_header['frame_rate']
		time_vector = numpy.arange(initial_frame,final_frame,dtype=numpy.float)
		time_vector = time_vector/frame_rate

		# Plot the motion signals
		# self.__plot_motion_signals(time_vector,signals)
		# self.__plot_motion_signals(time_vector,signals,('x','y','mag'))
		figure_handle = self.__plot_motion_signals(time_vector,signals,('x','y','mag'))
		figure_handle.savefig(pdf_file_name)

		# Save motion signals to HDF file
		# hdf_file_name = "motion_signals.hdf"
		self.__write_flow_signals_to_hdf_file(time_vector,signals,hdf_file_name)

		# Save motion signals to Excel file
		# excel_file_name = "motion_signals.xls"
		self.__write_excel_file(time_vector,signals,excel_file_name)

		# OK, we're done
		print "Motion signals have bee saved to the files %s.(hdf, xls, pdf)" % base_file_name

# ----------------------------------------------------------------------------------------------- #

	def __rearrange_motion_signals(self,sums,initial_frame,final_frame):

		# The number of frames to be processed
		n_frames = final_frame - initial_frame

		# The region and signal names
		region_names = sums[0].keys()
		signal_names = sums[0][region_names[0]].keys()

		# Initialize the variable 'signals' where the re-organized contents of
		# 'sums' will be stored
		signals = {}

		# Convert the variable 'sums' from a list of dictionaries into a dictionary whose
		# items are the regions of interest and whose sub-items are the various computed
		# signals (x, y, x_mag, y_mag, mag). The signals are stored as Numpy arrays
		for region in region_names:
			signals[region] = {}
			for signal in signal_names:
				# aux[region][signal] = numpy.zeros(flow_file.n_frames,dtype='float32')
				signals[region][signal] = numpy.zeros(n_frames,dtype='float64')
				# for frame_number in range(flow_file.n_frames):
				for frame_number in range(initial_frame,final_frame):
					indice = frame_number - initial_frame
					# aux[region][signal][frame_number] = sums[frame_number][region][signal]
					signals[region][signal][indice] = sums[indice][region][signal]

		# Return the variable 'signals'
		return signals

# ----------------------------------------------------------------------------------------------- #

	def __plot_motion_signals(self,time_vector,signals,signal_names=None):

		# The region and signal names
		region_names = signals.keys()
		region_names.sort()
		all_signal_names = signals[region_names[0]].keys()
		all_signal_names.sort()

		# If the input parameter 'signal_names' has not been provided,
		# it defaults to all signals
		if not signal_names:
			signal_names = all_signal_names

		# Make sure that 'signal_names' contains only valid signal names
		signal_names = [name for name in signal_names if name in all_signal_names]
		signal_names.sort()

		# The plot legends for all signals
		legendas = {'mag': 'Magnitude','x_mag': 'Horz. Magnitude','y_mag': 'Vert. Magnitude','x': 'Horizontal','y': 'Vertical'}

		# Discard legends for the signals that are not in 'signal_names'
		legendas = dict([(name, legendas[name]) for name in signal_names])

		# The number of regions
		n_regions = len(region_names)

		# The number of signals
		n_signals = len(signal_names)

		# The number of samples in the motion signals
		n_samples = signals[region_names[0]][signal_names[0]].shape[0]

		# A matrix with the signals to be plotted, one signal per column
		signal_matrix = numpy.zeros((n_samples,n_signals))

		# Initialize the axes handles vector
		axes_handles = [None]*n_regions

		# Create the figure where to plot the signals
		figure_handle = plt.figure()

		# Loop over regions, plotting the signals for each region in a separate panel (subplot)
		for m in range(n_regions):

			# Important: at least when using pyplot, the subplot number is one-based
			# (as opposed to Python's zero-based numbering system)
			# axes_handles[m] = plt.subplot(n_regions,1,m+1)
			axes_handles[m] = figure_handle.add_subplot(n_regions,1,m+1)
			# axes_handles[m] = figure_handle.add_subplot(n_regions,1,m+1,sharex=axes_handles[m-1])

			for n in range(n_signals):
				signal_matrix[:,n] = signals[region_names[m]][signal_names[n]]

			# Plot all signals for the current region on the current axes
			# plt.plot(time_vector,signal_matrix,'.-')
			axes_handles[m].plot(time_vector,signal_matrix,'.-')

			# Set the ylabel of the current axes
			#	ylabel(region_names{m},'Interpreter','none');
			axes_handles[m].set_ylabel(region_names[m])

			# Add a legend to the current axes
			axes_handles[m].legend([legendas[name] for name in signal_names])

			#	if (m<n_regions), set(gca,'XTickLabel',[]); end;
			if (m<n_regions-1):
				axes_handles[m].set_xticklabels("")

		#% Link all axes along the horizontal direction
		#linkaxes(axes_handles,'x');
		#
		# Set the xlabel for the last axes only
		axes_handles[-1].set_xlabel('Time (s)')

		# Show the figure on screen
		plt.show()

#		# Save the figure to a pdf file. This should go somewhere else, though
#		# (i.e., outside this method)
#		plt.savefig('teste.pdf')

		# Return the figure handle to the calling function
		return figure_handle

# ----------------------------------------------------------------------------------------------- #

	def __write_flow_signals_to_hdf_file(self,time_vector,signals,file_name):

		# The region and signal names
		region_names = signals.keys()
		region_names.sort()
		signal_names = signals[region_names[0]].keys()
		signal_names.sort()

		# Open the output file for writing
		fid = tables.openFile(file_name,"w")

		# Create a group for storing the flow signals
		sums_group = fid.createGroup("/", 'sums', 'Regions')

		# The array shape and type in the HDF file
		array_shape = time_vector.shape
		array_type = time_vector.dtype
		array_atom = tables.Atom.from_dtype(array_type)

		# CArray filters
		compression_method = "zlib"
		#compression_method = "bzip2"
		#compression_level = 0
		compression_level = 2
		array_filters = tables.Filters(complib=compression_method,complevel=compression_level)

		# Create a CArray for storing the time vector
		array = fid.createCArray("/",'time_vector',array_atom,array_shape,filters=array_filters)

		# Write the time vector to a CArray
		array[:] = time_vector

		# The plot legends for all signals
		legendas = {'mag': 'Magnitude','x_mag': 'Horz. Magnitude','y_mag': 'Vert. Magnitude','x': 'Horizontal','y': 'Vertical'}

		# The number of regions
		n_regions = len(region_names)

		# The number of signals
		n_signals = len(signal_names)

		# The number of samples in the motion signals
		# n_samples = signals[region_names[0]][signal_names[0]].shape[0]
		n_samples = len(time_vector)

		# Initialize a vector where to store the region groups
		region_groups = [None]*n_regions

		# Loop over regions, plotting the signals for each region in a separate panel (subplot)
		for m in range(n_regions):

			# Create a group in the HDF file for the current region
			region_groups[m] = fid.createGroup(sums_group,region_names[m],region_names[m])

			# Loop over signals in the current region
			for n in range(n_signals):

				# Create an array in the HDF file for the current signal
				array = fid.createCArray(region_groups[m],signal_names[n],array_atom,array_shape,filters=array_filters)

				# Write the current signal to the current CArray
				array[:] = signals[region_names[m]][signal_names[n]]

		# Close the file
		fid.close()

# ----------------------------------------------------------------------------------------------- #

	def __write_excel_file(self,time_vector,signals,file_name):

		# Create a work book
		work_book = xlwt.Workbook()

		# Add a sheet to the work book
		flow_sheet = work_book.add_sheet('flow motion')

		# The number of samples in the motion signals
		n_samples = len(time_vector)

		# The initial row in the Excel file where to start writing the motion signals.
		# Here's row the first rows in the sheet are used:
		# Row 0 -> signal name
		# Row 1 -> sub-signal name
		# Row 2 -> empty
		initial_table_row = 3

		# The number of the current column where to write data to
		column_number = 0

		# Write the name of the time vector signal to the Excel file
		flow_sheet.write(0,column_number,"Time (s)")

		# Loop over samples of the current signal
		for m in range(n_samples):

			# Write the current sample of the current signal to the output file
			flow_sheet.write(m+initial_table_row,column_number,time_vector[m])

		# The signal names, in alphabetical order
		signal_names = signals.keys()
		signal_names.sort()

		# Loop over signals
		for signal_name in signal_names:

			# The sub-signal names, in alphabetical order
			sub_signal_names = signals[signal_name].keys()
			sub_signal_names.sort()

			# Loop over sub-signals ('x', 'y', 'mag', etc)
			for sub_signal_name in sub_signal_names:

				# Increment the column number
				column_number = column_number+1

				# Write the signal name to the current column in the Excel sheet
				flow_sheet.write(0,column_number,signal_name)

				# Write the sub-signal name to the current column in the Excel sheet
				flow_sheet.write(1,column_number,sub_signal_name)

				# The current signal
				signal = signals[signal_name][sub_signal_name]

				# Number cells in Excel are kept as IEEE 754 64-bit floats (the same as the
				# Python float type). The xlwt module detects common types (int, long, etc)
				# and automatically convert them to float. For everything else, we have to
				# do the conversion ourselves.
				if (signal.dtype.type != numpy.float64):
					signal = signal.astype('float64')

				# Loop over samples of the current signal
				for m in range(signal.size):

					# Write the current sample of the current signal to the output file
					flow_sheet.write(m+initial_table_row,column_number,signal[m])

		# Save the file to disk
		work_book.save(file_name)

# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

# Create the Qt application
app = QtWidgets.QApplication(sys.argv)

# Create our application's window
flow_gui = FlowGui()
flow_gui.show()

# Enter the event loop
sys.exit(app.exec_())


#flow_reader = flow_lib.FlowReader("CL1JD1Test_frames_0_10.hdf")
## flow_reader.process_flow_file()
#flow_reader.run()


# ----------------------------------------------------------------------------------------------- #
# ----------------------------------------------------------------------------------------------- #

