#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

# ----------------------------------------------------------------------------------------------- #

import sys
import flow_lib

# ----------------------------------------------------------------------------------------------- #

# Author: Adriano Vilela Barbosa

def main():

	# The name of the video input file
	input_file_name = sys.argv[1]

	# The name of the output flow file (the big file with the optical flow velocity fields).
	# If "None", no output flow file will be saved. If "[]", an output flow file with a
	# default name (based on the input file's name and the initial and final frame numbers)
	# will be created.
	output_file_name = None

	# The frame range to be analyzed (from initial_frame to final_frame). If initial_frame
	# is None, the analysis will start from the first frame (frame zero) in the input video
	# file. If final_frame is None, the analysis will stop at the last frame in the input
	# video file. Here, our analysis range will be the whole input video.
	initial_frame = None
	final_frame = None

	# Create a MovieReaderOpencv object for handling the input video.
	# The only reason we need to do this here is to find the video's
	# resolution (frame size) in order to pass it to the function
	# responsible for creating the regions of interest/disinterest
	input_video = flow_lib.MovieReaderOpencv(input_file_name)

	# The video's frame size
	# frame_size = (720, 480)
	frame_size = (input_video.width, input_video.height)

	# Define the regions of interest. If flow_regions is None, a single
	# region of interest the size of the whole frame is defined
	# flow_regions = None
	flow_regions = set_regions(frame_size)

	# Construct the FlowApplication object
	flow_app = flow_lib.FlowApplication(
		input_file_name,
		flow_regions,
		output_file_name,
		initial_frame,
		final_frame)

	# Run the flow application
	flow_app.run()

# ----------------------------------------------------------------------------------------------- #

def set_regions(frame_size):

		# The videos' resolution
		# frame_size = (720, 480)

		# Create a FlowRegions object
		flow_regions = flow_lib.FlowRegions(frame_size)

		# IMPORTANT: REGION COORDINATES ARE SPECIFIED AS [X1, Y1, X2, Y2]

		# The 'girl' region of interest
		region_1 = {}
		region_1["name"] = "girl"
		region_1["type"] = "interest"
		region_1["position"] = [108, 42, 722, 516]

		# The 'background' region of interest
		region_2 = {}
		region_2["name"] = "background"
		region_2["type"] = "interest"
		region_2["position"] = [25, 390, 106, 516]

		# The regions of disinterest
		region_3 = {}
		region_3["name"] = "Ignored"
		region_3["type"] = "disinterest"
		region_3["position"] = [0, 0, 180, 64]

		region_4 = {}
		region_4["name"] = "Ignored"
		region_4["type"] = "disinterest"
		region_4["position"] = [618, 42, 722, 178]

		# Add regions of interest and/or disinterest to the FlowRegions object
		# flow_regions.add_region(name, type, position)
		flow_regions.add_region(region_1["name"], region_1["type"], region_1["position"])
		flow_regions.add_region(region_2["name"], region_2["type"], region_2["position"])
		flow_regions.add_region(region_3["name"], region_3["type"], region_3["position"])
		flow_regions.add_region(region_4["name"], region_4["type"], region_4["position"])

		# Return the FlowRegions objects
		return flow_regions

# ----------------------------------------------------------------------------------------------- #

if __name__ == "__main__":
	main()

# ----------------------------------------------------------------------------------------------- #
