#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------------------------- #

# Author: Adriano Vilela Barbosa

# Use a non-interactive matplotlib backend. We have to do this
# before importing flow_lib, since it will import matplotlib
# with an interactive backend
import matplotlib
matplotlib.use('Agg')

import sys, os
import flow_scripts

# ----------------------------------------------------------------------------------------------- #

def main():

	# Usage example:
	# python batch_flow_processing.py /home/adriano/videos/ .mov

	# Important:
	# - the file names must be relative to the folder name
	# - the extension must start with a dot (e.g., .avi, .mov, etc)

	# Take the directory name from the first input argument
	dir_name = sys.argv[1]

	# Take the file extension from the second input argument
	extension = sys.argv[2]

	# Check if the first input argument is a directory name
	if not os.path.isdir(dir_name):
		print "Error: the first input argument must be a directory."
		sys.exit(-1)

	# The list of files in the directory 'dir_name'. This does NOT include directory names
	file_list = [os.path.normcase(f) for f in os.listdir(dir_name) if os.path.isfile(os.path.join(dir_name,f))]

	# pdb.set_trace()

	# Loop over the files in the provided folder
	for file_name in file_list:

		(prefix,suffix) = os.path.splitext(file_name)

		# if suffix.lower() == ".mov":
		if suffix.lower() == extension:

			# The input file name (absolute path)
			file_name = os.path.join(dir_name,file_name)

			# Process the current file
			flow_scripts.perform_flow_analysis(file_name)

# ----------------------------------------------------------------------------------------------- #

if __name__ == "__main__":
	main()

# ----------------------------------------------------------------------------------------------- #
