#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author: Adriano Vilela Barbosa

# ----------------------------------------------------------------------------------------------- #

import sys, os
import argparse
import flow_scripts

# ----------------------------------------------------------------------------------------------- #

def define_command_line_options():

	# Create an argument parser object
	parser = argparse.ArgumentParser(description='Create optical flow movie.')

	# Input option for the input file
	parser.add_argument('input_file',help='input flow field (.hdf) file')

	# Input option for the output file
	parser.add_argument('-o',dest='output_file',type=str,help='output movie file (default: same name as input file [.mov])')

	# Input option for the xmin value
	parser.add_argument('--xmin',type=float,default=0,help='lower flow threshold (default: 0)')

	# Input option for the xmax value
	parser.add_argument('--xmax',type=float,default=1,help='upper flow threshold (default: 1)')

	# Input option for overwrite flag
	parser.add_argument('--overwrite', action='store_true',help='force overwrite of the output movie file')

	# Return the parser object
	return parser

# ----------------------------------------------------------------------------------------------- #

def check_input_arguments(args):

	# Check if the input file exists
	if not os.path.isfile(args.input_file):
		print "Error: input file '%s' not found." % args.input_file
		sys.exit(-1)

	# If the output file already exists and the --overwrite flag
	# has not been set, we print out an error message and exit
	if args.output_file:
		if (os.path.isfile(args.output_file) & (not args.overwrite)):
			print "Error: output file already exists. Use '--overwrite' to force overwrite."
			sys.exit(-1)

	# xmin must be a number between 0 and 1
	if ((args.xmin < 0) | (args.xmin > 1)):
		print "Error: xmin must be a number between 0 and 1."
		sys.exit(-1)

	# xmax must be a number between 0 and 1
	if ((args.xmax < 0) | (args.xmax > 1)):
		print "Error: xmax must be a number between 0 and 1."
		sys.exit(-1)

	# xmax must be greater than xmin
	if (args.xmin >= args.xmax):
		print "Error: xmax must be greater than xmin."
		sys.exit(-1)

# ----------------------------------------------------------------------------------------------- #

if __name__ == "__main__":

	# Define the valid command line options
	parser = define_command_line_options()

	# Parse the command line options provided by the user
	args = parser.parse_args()

	# Check the input arguments provided in the command line
	check_input_arguments(args)

	# Create the flow movie
	flow_scripts.create_flow_movie(args.input_file,args.output_file,args.xmin,args.xmax)

# ----------------------------------------------------------------------------------------------- #
