#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author: Adriano Vilela Barbosa

# ----------------------------------------------------------------------------------------------- #

import sys, os
import argparse
import flow_scripts

# ----------------------------------------------------------------------------------------------- #

def define_command_line_options():

	# Create an argument parser object
	parser = argparse.ArgumentParser(description='Perform optical flow analysis of a movie file.')

	# Input option for the input file
	parser.add_argument('input_file',help='input movie file')

	# Input option for the regions file
	parser.add_argument('-regions',dest='regions_file',type=str,default=None,help='excel file with regions of interest/disinterest')

	# Input option for the output file
	parser.add_argument('-o',dest='output_file',type=str,default=None,help='base name for the various output files (default: same name as input file)')

	# Input option for the initial frame value
	parser.add_argument('-i',dest='initial_frame',type=int,default=None,help='initial frame (default: first frame of the video)')

	# Input option for the initial frame value
	parser.add_argument('-f',dest='final_frame',type=int,default=None,help='final frame (default: last frame of the video)')

	# Return the parser object
	return parser

# ----------------------------------------------------------------------------------------------- #

def check_input_arguments(args):

	# Check if the input file exists
	if not os.path.isfile(args.input_file):
		print "Error: input file '%s' not found." % args.input_file
		sys.exit(-1)

	# The initial frame must be a non-negative integer
	if args.initial_frame is not None:
		if (args.initial_frame < 0):
			print "Error: the initial frame must be a non-negative integer."
			sys.exit(-1)

	# The final frame must be a non-negative integer
	if args.final_frame is not None:
		if (args.final_frame < 0):
			print "Error: the final frame must be a non-negative integer."
			sys.exit(-1)

	# The final frame must be greater than the initial frame
	if ((args.initial_frame is not None) & (args.final_frame is not None)):
		if (args.initial_frame >= args.final_frame):
			print "Error: the final frame must be greater than the initial frame."
			sys.exit(-1)

# ----------------------------------------------------------------------------------------------- #

def main():

	# Define the valid command line options
	parser = define_command_line_options()

	# Parse the command line options provided by the user
	args = parser.parse_args()

	# Check the input arguments provided in the command line
	check_input_arguments(args)

	# Process the movie file
	flow_scripts.perform_flow_analysis(args.input_file, args.regions_file, args.output_file, args.initial_frame, args.final_frame)

# ----------------------------------------------------------------------------------------------- #

if __name__ == "__main__":
	main()

# ----------------------------------------------------------------------------------------------- #
